@@ -1,6 +1,6 @@
 import socket
-
-import librabbitmq
+import functools
+import pika
 
 from clitellum import logger_manager
 
@@ -62,6 +62,20 @@ class AmqpMessageProperties:
 
     return properties
 
+  def to_pika_basic_properties(self):
+    prop = pika.BasicProperties(
+        content_type=self.content_type,
+        content_encoding=self.content_encoding,
+        headers=self.headers,
+        delivery_mode=self.delivery_mode,
+        priority=self.priority,
+        correlation_id=self.correlation_id,
+        reply_to=self.reply_to,
+        expiration=self.expiration,
+        app_id=self.app_id,
+        message_id=self.message_id)
+    return prop
+
   @classmethod
   def from_basic_properties(cls, properties):
     """
@@ -71,15 +85,16 @@ class AmqpMessageProperties:
         :return:
         """
     amp = AmqpMessageProperties()
-    amp.content_encoding = properties['content_encoding'] if 'content_encoding' in properties else None
-    amp.message_id = properties['message_id'] if 'message_id' in properties else None
-    amp.reply_to = properties['reply_to'] if 'reply_to' in properties else None
-    amp.app_id = properties['app_id'] if 'app_id' in properties else None
-    amp.expiration = properties['expiration'] if 'expiration' in properties else None
-    amp.correlation_id = properties['correlation_id'] if 'correlation_id' in properties else None
-    amp.priority = properties['priority'] if 'priority' in properties else None
-    amp.headers = properties['headers'] if 'headers' in properties else None
-    amp.content_type = properties['content_type'] if 'content_type' in properties else None
+    amp.content_encoding = properties.content_encoding
+    amp.message_id = properties.message_id
+    amp.reply_to = properties.reply_to
+    amp.app_id = properties.app_id
+    amp.expiration = properties.expiration
+    amp.correlation_id = properties.correlation_id
+    amp.priority = properties.priority
+    amp.headers = properties.headers
+    amp.content_type = properties.content_type
+
     return amp
 
   @classmethod
@@ -100,11 +115,11 @@ class AmqpMesageFrame:
         :return: AmqpMessageFrame
         """
     amf = AmqpMesageFrame()
-    amf.exchange = frame['exchange'] if 'exchange' in frame else None
-    amf.consumer_tag = frame['consumer_tag'] if 'consumer_tag' in frame else None
-    amf.delivery_tag = frame['delivery_tag'] if 'delivery_tag' in frame else None
-    amf.routing_key = frame['routing_key'] if 'routing_key' in frame else None
-    amf.redelivered = frame['redelivered'] if 'redelivered' in frame else None
+    amf.exchange = frame.exchange
+    amf.consumer_tag = frame.consumer_tag
+    amf.delivery_tag = frame.delivery_tag
+    amf.routing_key = frame.routing_key
+    amf.redelivered = frame.redelivered
     return amf
 
   def __init__(self):
@@ -118,16 +133,16 @@ class AmqpMesageFrame:
 class AmqpMessage:
 
   @classmethod
-  def create_from_message(cls, message):
+  def create_from_message(cls, method_frame, header_frame, body):
     """
         Crea una instancia de AmqpMessage a partir del mensaje recibido
         :param message:
         :return:
         :rtype: clitellum.channels.amqp.AmqpMessage
         """
-    amf = AmqpMesageFrame.from_message_frame(message.delivery_info)
-    amp = AmqpMessageProperties.from_basic_properties(message.properties)
-    return AmqpMessage(amf, amp, message.body.tobytes(), amf.routing_key)
+    amf = AmqpMesageFrame.from_message_frame(method_frame)
+    amp = AmqpMessageProperties.from_basic_properties(header_frame)
+    return AmqpMessage(amf, amp, body, amf.routing_key)
 
   def __init__(self, frame: AmqpMesageFrame, properties: AmqpMessageProperties, body: bytes, routing_key: str):
     self.frame = frame
@@ -146,23 +161,13 @@ class AmqpConnection:
     """
         :type: channels.configuration.ChannelConfiguration
         """
-
     self._connection = None
-    """
-        :type: amqp.connection.Connection
-        """
-
     self._isConnected = False
     self._channel = None
-    """
-        :type: librabbitmq.Channel
-        """
     self._closing = False
+    self._consuming = False
     self._consumer_tag = None
     self._channel_send = None
-    """
-        :type: librabbitmq.Channel
-        """
     self._mutex = threading.Lock()
     self._mutex_ack = threading.Lock()
 
@@ -193,15 +198,12 @@ class AmqpConnection:
       if self._connection is not None and self._connection.is_open:
         self._connection.close()
 
-      parameters = self._configuration.get_connection_parameters()
+      parameters = pika.URLParameters(self._configuration.uri)
+      parameters.heartbeat = self._configuration.heartbeat
 
       logger_manager.get_logger().debug('Creando conexion')
-      self._connection = librabbitmq.Connection(
-          host=parameters.host,
-          userid=parameters.userid,
-          password=parameters.password,
-          virtual_host=parameters.virtual_host,
-          heartbeat=self._configuration.heartbeat)
+
+      self._connection = pika.BlockingConnection(parameters=parameters)
 
       logger_manager.get_logger().debug('Creando channel')
       self._channel = self._connection.channel(1)
@@ -220,6 +222,8 @@ class AmqpConnection:
           self._channel.queue_bind(
               queue=self._configuration.queue.name, exchange=self._configuration.exchange.name, routing_key=key)
       self._channel.basic_qos(0, self._configuration.prefetch_count, False)
+      if self._configuration.confirm_sent:
+        self._channel_send.confirm_delivery()
 
     except Exception as ex:
       logger_manager.get_logger().error('Error Creando conexion', ex)
@@ -262,10 +266,16 @@ class AmqpConnection:
     self._mutex.acquire()
     while not self._closing:
       try:
-        properties = amqp_message_properties.to_dict()
-        self._channel_send.basic_publish((message, properties),
-                                         exchange=self._configuration.exchange.name,
-                                         routing_key=routing_key)
+        properties = amqp_message_properties.to_pika_basic_properties()
+        self._channel_send.basic_publish(
+            self._configuration.exchange.name,
+            routing_key,
+            message,
+            properties,
+            mandatory=self._configuration.confirm_delivery)
+        break
+      except pika.exceptions.UnroutableError as unroute_exception:
+        logger_manager.get_logger().warn("No es posible enrutar el mensaje %s" % routing_key)
         break
       except Exception as ex:
         logger_manager.get_logger().error("Error de conexion al enviar el mensaje %s" % ex)
@@ -275,15 +285,19 @@ class AmqpConnection:
     self._mutex.release()
 
   def consume(self):
-    self._consumer_tag = self._channel.basic_consume(queue=self._configuration.queue.name, callback=self.__read_message)
+    self._consuming = True
+    self._consumer_tag = self._channel.basic_consume(self._configuration.queue.name, self.__read_message)
     logger_manager.get_logger().info("Start Consuming %s", self._consumer_tag)
+    self._channel.start_consuming()
 
   def drain_events(self, timeout):
-    self._connection.drain_events(timeout=timeout)
+    # self._connection.drain_events(timeout=timeout)
+    pass
 
-  def __read_message(self, message):
-    logger_manager.get_logger().debug("Message Received %s", message.body.tobytes())
-    amqp_message = AmqpMessage.create_from_message(message)
+  # def __read_message(self, message):
+  def __read_message(self, channel, method_frame, header_frame, body):
+    logger_manager.get_logger().debug("Message Received %s", str(body, encoding='utf-8'))
+    amqp_message = AmqpMessage.create_from_message(method_frame, header_frame, body)
     self.on_message_received(amqp_message)
 
   def ack(self, message: AmqpMessage):
@@ -294,11 +308,21 @@ class AmqpConnection:
 
     try:
       self._mutex_ack.acquire()
-      self._channel.basic_ack(delivery_tag=message.frame.delivery_tag, multiple=False)
+      cb = functools.partial(self._ack_message, self._channel, message.frame.delivery_tag)
+      self._connection.add_callback_threadsafe(cb)
     except Exception as ex:
       logger_manager.get_logger().error("Error al enviar el ack", ex)
     finally:
       self._mutex_ack.release()
 
+  def _ack_message(self, channel, delivery_tag):
+    channel.basic_ack(delivery_tag=delivery_tag, multiple=False)
+
   def basic_cancel(self):
-    self._channel.basic_cancel(self._consumer_tag)
+    self._consuming = False
+    # self._channel.basic_cancel(self._consumer_tag)
+    cb = functools.partial(self._basic_cancel, self._channel, self._consumer_tag)
+    self._connection.add_callback_threadsafe(cb)
+
+  def _basic_cancel(self, channel, consumer_tag):
+    channel.stop_consuming(consumer_tag)
